Index: src/mips64/code-stubs-mips64.cc |
diff --git a/src/mips64/code-stubs-mips64.cc b/src/mips64/code-stubs-mips64.cc |
index e570447e8f9cce120fe72e541d106c607c300c74..0968862c7d55c99b81b2a6634a3052b28c385a0b 100644 |
--- a/src/mips64/code-stubs-mips64.cc |
+++ b/src/mips64/code-stubs-mips64.cc |
@@ -1419,7 +1419,7 @@ void RegExpExecStub::Generate(MacroAssembler* masm) { |
// (6) External string. Make it, offset-wise, look like a sequential string. |
// Go to (4). |
// (7) Short external string or not a string? If yes, bail out to runtime. |
// (8) Sliced string. Replace subject with parent. Go to (1). |
Label check_underlying; // (1) |
Label seq_string; // (4) |
@@ -1443,6 +1443,7 @@ void RegExpExecStub::Generate(MacroAssembler* masm) { |
// (2) Sequential or cons? If not, go to (5). |
STATIC_ASSERT(kConsStringTag < kExternalStringTag); |
STATIC_ASSERT(kSlicedStringTag > kExternalStringTag); |
+ STATIC_ASSERT(kThinStringTag > kExternalStringTag); |
STATIC_ASSERT(kIsNotStringMask > kExternalStringTag); |
STATIC_ASSERT(kShortExternalStringTag > kExternalStringTag); |
// Go to (5). |
@@ -1469,12 +1470,12 @@ void RegExpExecStub::Generate(MacroAssembler* masm) { |
__ Branch(&runtime, ls, a3, Operand(a1)); |
__ SmiUntag(a1); |
- STATIC_ASSERT(kStringEncodingMask == 4); |
- STATIC_ASSERT(kOneByteStringTag == 4); |
+ STATIC_ASSERT(kStringEncodingMask == 8); |
+ STATIC_ASSERT(kOneByteStringTag == 8); |
STATIC_ASSERT(kTwoByteStringTag == 0); |
__ And(a0, a0, Operand(kStringEncodingMask)); // Non-zero for one_byte. |
__ ld(t9, FieldMemOperand(regexp_data, JSRegExp::kDataOneByteCodeOffset)); |
- __ dsra(a3, a0, 2); // a3 is 1 for one_byte, 0 for UC16 (used below). |
+ __ dsra(a3, a0, 3); // a3 is 1 for one_byte, 0 for UC16 (used below). |
__ ld(a5, FieldMemOperand(regexp_data, JSRegExp::kDataUC16CodeOffset)); |
__ Movz(t9, a5, a0); // If UC16 (a0 is 0), replace t9 w/kDataUC16CodeOffset. |
@@ -1720,12 +1721,18 @@ void RegExpExecStub::Generate(MacroAssembler* masm) { |
__ And(at, a1, Operand(kIsNotStringMask | kShortExternalStringMask)); |
__ Branch(&runtime, ne, at, Operand(zero_reg)); |
- // (8) Sliced string. Replace subject with parent. Go to (4). |
+ // (8) Sliced or thin string. Replace subject with parent. Go to (4). |
+ Label thin_string; |
+ __ Branch(&thin_string, eq, a1, Operand(kThinStringTag)); |
// Load offset into t0 and replace subject string with parent. |
__ ld(t0, FieldMemOperand(subject, SlicedString::kOffsetOffset)); |
__ SmiUntag(t0); |
__ ld(subject, FieldMemOperand(subject, SlicedString::kParentOffset)); |
__ jmp(&check_underlying); // Go to (1). |
+ |
+ __ bind(&thin_string); |
+ __ ld(subject, FieldMemOperand(subject, ThinString::kActualOffset)); |
+ __ jmp(&check_underlying); // Go to (1). |
#endif // V8_INTERPRETED_REGEXP |
} |
@@ -1419,7 +1419,7 @@ void RegExpExecStub::Generate(MacroAssembler* masm) { |
// Go to (4). |
// (7) Short external string or not a string? If yes, bail out to runtime. |
- // (8) Sliced string. Replace subject with parent. Go to (1). |
+ // (8) Sliced or thin string. Replace subject with parent. Go to (1). |
Label check_underlying; // (1) |
Label seq_string; // (4) |